﻿using Microsoft.AspNetCore.Mvc;
using StubLib;
using Web_Api.Mapper;

namespace Web_Api.Controllers
{
    [ApiController]
    [Route("api/[controller]")]
    public class SkinsController : Controller
    {
        private StubData.SkinsManager SkinsManager { get; set; } = new StubData.SkinsManager(new StubData());

        private readonly ILogger<SkinsController> _logger;

        public SkinsController(ILogger<SkinsController> logger)
        {
            _logger = logger;
        }

        [HttpGet]
        public async Task<IActionResult> Get()
        {
            var list = await SkinsManager.GetItems(0, await SkinsManager.GetNbItems());
            return Ok(list.Select(skin => skin?.toDTO()));
        }

        [HttpGet("name")]
        public async Task<IActionResult> GetById(string name)
        {
            var skinSelected = await SkinsManager.GetItemsByName(name, 0, await SkinsManager.GetNbItemsByName(name), null);
            return Ok(skinSelected.Select(skin => skin?.toDTO()));
        }

        [HttpPost("addSkin")]
        public async Task<IActionResult> AddSkin(SkinDTO skin)
        {
            var newSkin = skin.toModel();
            await SkinsManager.AddItem(newSkin);
            if(skin.Description == "string")
            {
                skin.Description = "Aucune bio";
            }
            Console.WriteLine("Le skin { " + skin.Name + " } avec pour description { " + skin.Description + " } a bien été ajouté");
            return Ok(newSkin);
        }

        [HttpPut("updateSkin")]
        public async Task<IActionResult> UpdateChampion(string name, SkinDTO skin)
        {
            var skinSelected = await SkinsManager.GetItemsByName(name, 0, await SkinsManager.GetNbItemsByName(name), null);
            var existingSkin = skinSelected.FirstOrDefault();
            if(existingSkin == null)
            {
                Console.WriteLine("Le skin { " + name + " } n'existe pas !");
                return NotFound();
            }

            var updatedSkin = skin.toModel();
            await SkinsManager.UpdateItem(existingSkin, updatedSkin);
            if(skin.Description == "string")
            {
                skin.Description = "Aucune bio";
            }
            Console.WriteLine("Le skin { " + name + " } a été modifié en " + " { " + updatedSkin.Name + " } avec pour description { " + updatedSkin.Description + " }<");
            return Ok();
        }

        [HttpDelete("deleteSkin")]
        public async Task<IActionResult> DeleteChampion(string name)
        {
            var skinSelected = await SkinsManager.GetItemsByName(name, 0, await SkinsManager.GetNbItemsByName(name), null);
            if (!await SkinsManager.DeleteItem(skinSelected.FirstOrDefault()))
            {
                Console.WriteLine("skin { " + name + " } non trouvé !");
                return NotFound();
            }
            Console.WriteLine("skin { " + name + " } supprimé");
            return Ok();
        }
        /*// GET: SkinsController
        public ActionResult Index()
        {
            return View();
        }

        // GET: SkinsController/Details/5
        public ActionResult Details(int id)
        {
            return View();
        }

        // GET: SkinsController/Create
        public ActionResult Create()
        {
            return View();
        }

        // POST: SkinsController/Create
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(IFormCollection collection)
        {
            try
            {
                return RedirectToAction(nameof(Index));
            }
            catch
            {
                return View();
            }
        }

        // GET: SkinsController/Edit/5
        public ActionResult Edit(int id)
        {
            return View();
        }

        // POST: SkinsController/Edit/5
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(int id, IFormCollection collection)
        {
            try
            {
                return RedirectToAction(nameof(Index));
            }
            catch
            {
                return View();
            }
        }

        // GET: SkinsController/Delete/5
        public ActionResult Delete(int id)
        {
            return View();
        }

        // POST: SkinsController/Delete/5
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Delete(int id, IFormCollection collection)
        {
            try
            {
                return RedirectToAction(nameof(Index));
            }
            catch
            {
                return View();
            }
        }*/
    }
}
